#!/usr/bin/perl

#Version 1.00

# For each line of the log file
while (my $line = <>) {

	chomp $line;
	
	if ($line =~ /CLICK\t/) { # It was a click

		# Parse the click record
		my ($skip, $date, $query, $ip, $url) = split /\t/, $line;

		# Get the results the user was last shown for this query
		my ($qDate, $teams, $urlsString) = split /\|/, $results{"$query:$ip"};

		# If this URL was in the results, count the click
		my @urls = split /;/, $urlsString;
		for (my $pos=0; $pos<=$#urls; $pos++) 
		{
			if ($urls[$pos] eq $url) 
			{
				# Record which team and positon the clicks were on
				my $team = substr ($teams, $pos, 1);
				$clicks{"$query:$ip:$qDate"} .= "$team($pos)";

				# Keep a running count of how many clicks each team got
				my $score = 0;
				if ($team eq "A") { $score = 1; }
				if ($team eq "B") { $score = -1; }
				$scores{"$query:$ip:$qDate"} += $score;
			}
		}

	} else { # It was a query
	
		# Parse the query record
		my ($skip, $date, $query, $ip, $teams, $urls, $orig, $new) = split /\t/, $line;
		
		# Just keep track of the last time the query was issued
		$results{"$query:$ip"} .= "$date|$teams|$urls";
	}
}


# Print a summary of the results
my $A_tot = 0;
my $B_tot = 0;
foreach my $key (keys %clicks) {
	
	my $winner = "-";
	if ($scores{$key} > 0) { $winner = "A"; $A_tot++;}
	if ($scores{$key} < 0) { $winner = "B"; $B_tot++;}
	print "$winner\t$key\t$clicks{$key}\n";
}
print "\n";
print "A total: $A_tot\n";
print "B total: $B_tot\n";


# Sign test 
my $p =  $A_tot / ($A_tot + $B_tot);
my $sigma = sqrt($p*(1-$p)/($A_tot + $B_tot));

# 1.96 for 95% confidence
my $lower = $p - 1.96*$sigma;
my $upper = $p + 1.96*$sigma;

print "mean: $p\n";
print "95% confidence interval: [$lower, $upper]\n";
if($upper < 0.5)
{
   print "B is better with 95% confidence\n";
}
elsif($lower > 0.5)
{
   print "A is better with 95% confidence\n";
}
else
{
   print "The two rankers cannot be distinguished with 95% confidence\n";
}


# Boostrapping to test consistency 
print "\n";
print "Bootstrap statistics:\n";
my @all_keys = keys %clicks;
my $tot = @all_keys;
my $bag_size = $tot;
my $num_bags = 500;

my $A_wins_bag = 0;
my $B_wins_bag = 0;
my $tie_bag = 0;

# Sampling loop
foreach my $i (1..$num_bags)
{
   my $A_tot = 0;
   my $B_tot = 0;

   # generating a sample
   foreach my $j (1..$bag_size)
   {
      my $ind = int(rand()*$tot);
      my $key = $all_keys[$ind];
      if ($scores{$key} > 0) { $A_tot++;}
      if ($scores{$key} < 0) { $B_tot++;}
   }

   # scoring sample
   if($A_tot > $B_tot)
   {
      $A_wins_bag++;
   }
   elsif($A_tot < $B_tot)
   {
      $B_wins_bag++;
   }
   else
   {
      $tie_bag++;
   }
}

print "Bootstrapped $num_bags bags of size $bag_size\n";
print "A won $A_wins_bag times\n";
print "B won $B_wins_bag times\n";
print "Tied $tie_bag times\n";


